import numpy as np
import cv2
from affineTransformTools import getTranslationX, getTranslationY, getRotation, getAffineTransform

video = cv2.VideoCapture('../record2.avi')
fps = video.get(cv2.CAP_PROP_FPS)
hasFrame, frame = video.read()
prev_gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

out = cv2.VideoWriter()
out.open('../record3.avi',cv2.VideoWriter_fourcc('M','J','P','G'),fps,(frame.shape[1],frame.shape[0]))

x = 0.0
y = 0.0
f = 0.0

while True:

    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    
    prev_points = cv2.goodFeaturesToTrack(prev_gray,maxCorners=200,qualityLevel=0.01,minDistance=30,blockSize=3)
    
    points, status, err = cv2.calcOpticalFlowPyrLK(prev_gray, gray, prev_points, None) 

    indices = np.where(status==1)[0]
    warp_matrix, _ = cv2.estimateAffinePartial2D(prev_points[indices], points[indices], method=cv2.LMEDS)
    
    dx = getTranslationX(warp_matrix)
    dy = getTranslationY(warp_matrix)
    df = getRotation(warp_matrix)
    
    x += dx
    y += dy
    f += df
    
    final_warp_matrix = getAffineTransform(1.0,1.0,f,-x,-y)
   
    stabilized = cv2.warpAffine(frame,final_warp_matrix,(frame.shape[1],frame.shape[0]))
    
    cv2.imshow("video",cv2.hconcat([frame,stabilized]))
    key = cv2.waitKey(1)
    if key == 27:
        break
        
    out.write(stabilized)
    
    hasFrame, frame = video.read()
    if not hasFrame:
        break

    prev_gray = gray
    
out.release()
cv2.destroyAllWindows()
